'From Squeak2.9alpha of 6 July 2000 [latest update: #2518] on 27 August 2000 at 9:12:33 am'!
"Change Set:		PPMReadWriter-nk
Date:				27 August 2000
Author:			Ned Konz

This is my second cut at a PPM reader/writer.

It will write a 256x256x32 Form out to a file in less than 3 seconds on my machine.
It may be somewhat slower on 16 and 8 bit color depths.

An off-by-one bug was fixed in the prior version.

Currently, it only reads rawbits (P6) images.

I'm putting it out so that it can be used in the ICFP Programming Contest."!

ImageReadWriter subclass: #PPMReadWriter
	instanceVariableNames: 'raw comments width height maxColorValue '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Files'!

!PPMReadWriter commentStamp: 'nk 8/26/2000 22:15' prior: 0!
This reads and writes PPM image files.
It reads both ASCII (P3) and Raw (P6) image files,
and currently only writes Raw (P6) files.

It writes out 32-bit Forms much faster than 16 or 8 bit Forms.

NOTE: this version does not correctly read P6 files, and will
not read P3 files at all.

Contributed by Ned Konz (ned@bike-nomad.com) late August 2000.!

!PPMReadWriter methodsFor: 'private' stamp: 'nk 8/26/2000 22:11'!
bytesFrom: word depth: depth into: byteArray 
	"Unpack word into r,g,b values, putting them into byteArray."

	depth = 32
		ifTrue: [byteArray
				at: 1
				put: ((word bitShift: -16)
						bitAnd: 255).
			byteArray
				at: 2
				put: ((word bitShift: -8)
						bitAnd: 255).
			byteArray
				at: 3
				put: (word bitAnd: 255)]
		ifFalse: [| color | 
			color _ Color colorFromPixelValue: word depth: depth.
			byteArray at: 1 put: (color red * 255) asInteger.
			byteArray at: 2 put: (color green * 255) asInteger.
			byteArray at: 3 put: (color blue * 255) asInteger]! !

!PPMReadWriter methodsFor: 'private' stamp: 'nk 8/26/2000 22:46'!
readHeader
	"Set width and height, and position stream at start of bytes"
	| magic |
	stream ascii.
	magic _ stream next: 2.
	raw _ (magic = 'P6').
	self skipCommentsAndSeparators.
	width _ Integer readFrom: stream.
	self skipCommentsAndSeparators.
	height _ Integer readFrom: stream.
	self skipCommentsAndSeparators.
	maxColorValue _ Integer readFrom: stream.
	raw
		ifTrue: [stream binary; next]
		ifFalse: [self skipCommentsAndSeparators]! !

!PPMReadWriter methodsFor: 'private' stamp: 'nk 8/26/2000 20:52'!
skipCommentsAndSeparators
	stream skipSeparators.
	[stream peekFor: $#]
		whileTrue: [stream skipSeparators.
			comments
				addLast: (stream upToAny: String crlf).
			stream skipSeparators]! !

!PPMReadWriter methodsFor: 'private' stamp: 'nk 8/26/2000 20:47'!
writeComments
	comments
		do: [:ea | self nextPutAll: '# ';
				 nextPutAll: ea;
				 lf]! !

!PPMReadWriter methodsFor: 'private' stamp: 'nk 8/26/2000 20:51'!
writeHeader
	"Assumes raw output for now"
	self nextPutAll: 'P6';
		 lf.
	self writeComments.
	self nextPutAll: width printString;
		 space;
		 nextPutAll: height printString;
		 lf.
	self nextPutAll: '255';
		 lf! !

!PPMReadWriter methodsFor: 'private' stamp: 'nk 8/27/2000 08:45'!
writeRawBodyFor: aForm 
	| hibernated byteArray peeker depth |
	hibernated _ aForm unhibernate.
	byteArray _ ByteArray new: 3.
	depth _ aForm depth.
	peeker _ BitBlt current bitPeekerFromForm: aForm.
	0
		to: height - 1
		do: [:y | 0
				to: width - 1
				do: [:x | 
					self
						bytesFrom: (peeker pixelAt: x @ y)
						depth: depth
						into: byteArray.
					self nextPutAll: byteArray]].
	hibernated
		ifTrue: [aForm hibernate]! !

!PPMReadWriter methodsFor: 'TODO' stamp: 'nk 8/26/2000 19:19'!
readAsciiBody
	self isThisEverCalled! !

!PPMReadWriter methodsFor: 'TODO' stamp: 'nk 8/27/2000 09:07'!
readRawBody
	"Stream is positioned at start of bytes. Return Form with bytes."
	| form rowBytes line pixelLine pixIndex rgb |
	stream binary.
	form _ Form extent: width @ height depth: 32.
	rowBytes _ width * 3.
	line _ Form extent: width @ 1 depth: 32.
	0
		to: height - 1
		do: [:i | 
			pixelLine _ stream next: rowBytes.
			pixIndex _ 1.
			1
				to: width
				do: [:j | 
					rgb _ ((pixelLine at: pixIndex)
								bitShift: 16)
								+ ((pixelLine at: pixIndex + 1)
										bitShift: 8)
								+ (pixelLine at: pixIndex + 2).
					line bits at: j put: rgb.
					pixIndex _ pixIndex + 3].
			form
				copy: line boundingBox
				from: line
				to: 0 @ i
				rule: Form over].
	^ form! !

!PPMReadWriter methodsFor: 'initialize-release' stamp: 'nk 8/27/2000 08:49'!
initialize
	width _ height _ 0.
	raw _ true.
	comments _ OrderedCollection new! !

!PPMReadWriter methodsFor: 'accessing' stamp: 'nk 8/26/2000 19:38'!
addComment: aString
	comments addLast: aString! !

!PPMReadWriter methodsFor: 'accessing' stamp: 'nk 8/26/2000 21:12'!
nextImage
	| form |
	"Read in the next PPM image from the stream."
	stream reset.
	stream binary.
	self readHeader.
	form _ raw
		ifTrue: [self readRawBody]
		ifFalse: [self readAsciiBody].

"Fixup the alpha channel in the 32bit form"
	(BitBlt current toForm: form) destRect: form boundingBox;
		fillColor: (Bitmap with: 4278190080);
		 combinationRule: 7;
		 copyBits.

	^form! !

!PPMReadWriter methodsFor: 'accessing' stamp: 'nk 8/26/2000 20:50'!
nextPutImage: aForm 
	height _ aForm height.
	width _ aForm width.
	raw _ true.
	"For now"
	maxColorValue _ 255.
	stream reset; binary.
	self writeHeader.
	self writeRawBodyFor: aForm! !

!PPMReadWriter methodsFor: 'accessing' stamp: 'nk 8/26/2000 20:46'!
understandsImageFormat
	"Test to see if the image stream format is understood by this  
	decoder.	This should be implemented in each subclass of  
	ImageReadWriter so that	a proper decoder can be selected  
	without ImageReadWriter having to know about all possible  
	image file types."
	| first |
	first _ (self next: 2) asString.
	^ first = 'P6'
		or: [first = 'P3']! !


!PPMReadWriter class methodsFor: 'image reading/writing' stamp: 'nk 8/26/2000 22:09'!
putForm: aForm onFileNamed: fileName withComment: commentString 
	"Store the given form on a file of the given name.
	Add commentString as a comment in the header."
	| writer |
	writer _ self on: (FileStream newFileNamed: fileName) binary.
	writer addComment: commentString.
	Cursor write
		showWhile: [writer nextPutImage: aForm].
	writer close! !

!PPMReadWriter class methodsFor: 'image reading/writing' stamp: 'nk 8/26/2000 22:08'!
putForm: aForm onFileNamed: fileName withComments: commentStrings
	"Store the given form on a file of the given name.
	Add each string in commentStrings to the file as a comment in the header."
	| writer |
	writer _ self on: (FileStream newFileNamed: fileName) binary.
	commentStrings do: [ :ea | writer addComment: ea ].
	Cursor write
		showWhile: [writer nextPutImage: aForm].
	writer close! !

!PPMReadWriter class methodsFor: 'image reading/writing' stamp: 'nk 8/26/2000 22:08'!
putForm: aForm onStream: aWriteStream withComment: commentString 
	"Store the given form on a file of the given name.
	Add commentString as a comment in the header."
	| writer |
	writer _ self on: aWriteStream.
	writer addComment: commentString.
	Cursor write
		showWhile: [writer nextPutImage: aForm].
	writer close! !

!PPMReadWriter class methodsFor: 'image reading/writing' stamp: 'nk 8/26/2000 22:09'!
putForm: aForm onStream: aWriteStream withComments: commentStrings
	"Store the given form on a file of the given name. 
	Add each string in commentStrings as a comment in the header."
	| writer |
	writer _ self on: aWriteStream.
	commentStrings do: [ :ea | writer addComment: ea ].
	Cursor write
		showWhile: [writer nextPutImage: aForm].
	writer close! !

!PPMReadWriter class methodsFor: 'instance creation' stamp: 'nk 8/26/2000 18:18'!
new
	^super new initialize! !


!PositionableStream methodsFor: 'accessing' stamp: 'nk 8/26/2000 19:31'!
upToAny: aCollection 
	| newStream char |
	newStream _ WriteStream
				on: (String new: 100).
	[(char _ self next) isNil
		or: [ aCollection includes: char]]
		whileFalse: [newStream nextPut: char].
	^ newStream contents! !


!PPMReadWriter class reorganize!
('image reading/writing' putForm:onFileNamed:withComment: putForm:onFileNamed:withComments: putForm:onStream:withComment: putForm:onStream:withComments:)
('instance creation' new)
!

