'From Squeak3.2gamma of 15 January 2002 [latest update: #4889] on 20 August 2002 at 6:17:22 pm'!
"Change Set:		GraphicAnalyzer
Date:			20 August 2002
Author:			Eddie Cottongim

This is a factored version of the SpectrumAnalyzer. The main improvement is that it is easier and cleaner to add new analysis modes by subclassing GraphicAnalyzerModule. To use, create a GraphicAnalyzerMorph from the 'new morph' menu.

Modules provided: Signal, Spectrum, Sonogram, Frequency Finder, NoteFinder/Tuner.

The first three are taken from the original SpectrumAnalyzer; the latter due were helped greatly by John Maloney's frequency finding code.

This had been laying around for some time and I thought someone might be able to use it. I've also worked on a Constant-Q mode, which was functional but ugly and requires a large amount of support code that most people wouldn't want. I'll ressurect this if there is interest.

Feedback appreciated.

"!

Object subclass: #GraphicAnalyzerModule
	instanceVariableNames: 'display samplingRate '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-GraphicAnalyzer'!

!GraphicAnalyzerModule commentStamp: '<historical>' prior: 0!
Abstract. I provide services for SoundAnalyzerMorph: I will respond to buffer processing requests, and I provide a display. Take a look at SignalModule for a simple, concrete example. 

Structure:
 display			Morph -- visual feedback is provided here

Implementors should override most of my methods. For your contrete subclasses, be sure that on the class side, includeInAnalyzerMenu returns true.

I can trigger some events with self trigger: #eventName.
	#resetDisplay 	indicate my display has been recreated and clients need to ask me for a new 					one
		!

GraphicAnalyzerModule subclass: #FFTModule
	instanceVariableNames: 'fft '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-GraphicAnalyzer'!

!FFTModule commentStamp: '<historical>' prior: 0!
Abstract. I am the parent of SpectrumAnalyzerModules that use a FFT. I maintain a FFT for my children to use, and I provide the menu options to change FFT size.

Structure:
 fft				FFT -- My subclasses use this for processing.
!

FFTModule subclass: #FrequencyFinderModule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-GraphicAnalyzer'!

!FrequencyFinderModule commentStamp: '<historical>' prior: 0!
I determine the dominant frequency in the signal and display it in hz. The method was originally done by John Maloney. Thanks John!!

!

GraphicAnalyzerModule class
	instanceVariableNames: ''!
FFTModule class
	instanceVariableNames: ''!
FrequencyFinderModule class
	instanceVariableNames: ''!
BorderedMorph subclass: #GraphicAnalyzerMorph
	instanceVariableNames: 'soundInput statusLight levelMeter display uiColors activeModule '
	classVariableNames: 'UIColors '
	poolDictionaries: ''
	category: 'Morphic-GraphicAnalyzer'!

!GraphicAnalyzerMorph commentStamp: '<historical>' prior: 0!
This is a factored version of Squeak's original SpectrumAnalyzerMorph by John Maloney.
I provide a basic gui and input control. Actual signal analysis takes place in subclasses of SpectrumAnalyzerModule.

soundInput		SoundInputStream -- The signal source
display			Morph -- The visual display provided by the activeModule
activeModule	SpectrumAnalyzerModule -- The currently loaded module!

GraphicAnalyzerMorph class
	instanceVariableNames: ''!
FrequencyFinderModule subclass: #NoteFinderModule
	instanceVariableNames: 'calibrationFreq textDisplay meter lastValue '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-GraphicAnalyzer'!

!NoteFinderModule commentStamp: '<historical>' prior: 0!
I determine the strongest note and how well in tune it is (in equal temperment). 

Structure:
 calibrationFreq		Integer -- 	The frequency in hz for the note A2. Default and common standard is 440. Some musical groups tune as high as 445. Very old music (12th century) may be much lower, half an octave or more.

I operate by getting the strongest frequency from my parent and converting this to a note name.

The meter is "weighted" to make it swing less wildly. See processBuffer:.!

NoteFinderModule class
	instanceVariableNames: ''!
GraphicAnalyzerModule subclass: #SignalModule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-GraphicAnalyzer'!

!SignalModule commentStamp: '<historical>' prior: 0!
I show snapshots of the raw signal data as it arrives.

If I lag behind, I will drop buffers to keep up.!

SignalModule class
	instanceVariableNames: ''!
FFTModule subclass: #SonogramModule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-GraphicAnalyzer'!

!SonogramModule commentStamp: '<historical>' prior: 0!
I show a scrolling plot of the frequency spectrum over time, where the vertical axis is frequency, the horizontal axis is time, and amount of energy at a given frequency is shown as a grayscale value with larger values being darker.

I process all buffers, even if I get behind. If I lag too much try a smaller FFT size.!

SonogramModule class
	instanceVariableNames: ''!
FFTModule subclass: #SpectrumModule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-GraphicAnalyzer'!

!SpectrumModule commentStamp: '<historical>' prior: 0!
I show the frequency spectrum of the signal data as it arrives.!

SpectrumModule class
	instanceVariableNames: ''!

!GraphicAnalyzerModule methodsFor: 'interface' stamp: 'efc 11/29/2001 11:11'!
addCustomMenuItemsTo: aMenu
"aMenu is a CustomMenu. If I would like my own items added to the list, I should do it here. "

"This default implementation does nothing."! !

!GraphicAnalyzerModule methodsFor: 'interface' stamp: 'efc 11/29/2001 11:16'!
getDisplayMorph
"Return the morph that is my visual display. Should only be called once - I only expect to work with one display "

^display! !

!GraphicAnalyzerModule methodsFor: 'interface' stamp: 'efc 11/28/2001 16:02'!
getName
"What would I like to be called (like in a menu)"
^self class getName! !

!GraphicAnalyzerModule methodsFor: 'interface' stamp: 'efc 12/1/2001 15:57'!
setSamplingRate: rateInHz
"Take any desired action here. Some modules will want to reset their display with
self trigger: #resetDisplay
This default implementation stores the sampling rate only."

samplingRate _ rateInHz.
! !

!GraphicAnalyzerModule methodsFor: 'private' stamp: 'efc 11/28/2001 12:01'!
processBuffer: buf
"Process a single buffer. This method is not required and not used directly by the SoundAnalyzerMorph. It typically will be private. It is a convenient place to put the processing for single buffers. See processBuffersFrom: for the normally used public interface."

self subclassResponsibility! !

!GraphicAnalyzerModule methodsFor: 'processing' stamp: 'efc 11/28/2001 17:01'!
bufferSize
"What size buffers do I want to process? Size is in samples"

self subclassResponsibility ! !

!GraphicAnalyzerModule methodsFor: 'processing' stamp: 'efc 11/29/2001 14:12'!
processBuffersFrom: aSoundInputStream
"Process buffers from aSoundInputStream. Implementers may choose their own policy as to whether every buffer or only the most recent should be processed, as to not fall behind. "

"This default implentation processes only the most recent buffer."

| buf |
	aSoundInputStream bufferCount = 0 ifTrue: [^ self].

	[aSoundInputStream bufferCount > 0] whileTrue: [
		"skip to the most recent buffer"
		buf _ aSoundInputStream nextBufferOrNil].
	^ self processBuffer: buf.! !

!GraphicAnalyzerModule methodsFor: 'initializing' stamp: 'efc 11/29/2001 14:03'!
initialize
"Do any initialization here"! !


!FFTModule methodsFor: 'initializing' stamp: 'efc 11/29/2001 11:23'!
initialize

fft _ FFT new setSize: 256.! !

!FFTModule methodsFor: 'processing' stamp: 'efc 11/29/2001 14:02'!
bufferSize
^fft n! !

!FFTModule methodsFor: 'fft' stamp: 'efc 12/14/2001 05:25'!
setFFTSize
	"Set the size of the FFT used for frequency analysis. return nil if no change made"

	| aMenu sz |
	aMenu _ CustomMenu new title: 'FFT size (currently ', fft n printString, ')'.
	((7 to: 11) collect: [:n | 2 raisedTo: n]) do:[:r | aMenu add: r printString action: r].
	sz _ aMenu startUp.
	sz ifNil: [^ nil].

	fft _ FFT new: sz.
! !

!FFTModule methodsFor: 'interface' stamp: 'efc 11/29/2001 14:49'!
addCustomMenuItemsTo: aMenu

aMenu add: 'set FFT Size' action: #setFFTSize.! !


!FrequencyFinderModule methodsFor: 'processing' stamp: 'efc 11/30/2001 17:00'!
findDominantFrequencyWindowSize: windowSize fft: aFFT
	| realData imagData |

	realData _ aFFT realData.
	imagData _ aFFT imagData.
	
	^ self findDominantFrequencyWindowSize: windowSize real: realData imag: imagData.
! !

!FrequencyFinderModule methodsFor: 'processing' stamp: 'efc 11/29/2001 17:34'!
findDominantFrequencyWindowSize: windowSize real: real imag: imag
| numSamples n ampl i low maxWeight centerFreq i1 i2 moment weight |
"Due to Don Cross - see class comment.
    int numSamples,        	total number of frequency samples in buffer
    int windowSize,          	width of search window (experiment with this; try 4)
    double *real,            	real coefficient buffer
    double *imag,             	imaginary coefficient buffer
    double *ampl,             	work buffer for holding r*r+i*i values
    double samplingRate )   	sampling rate expressed in samples/second [Hz]
"
numSamples _ self bufferSize.
"number of positive frequencies."
n _ numSamples / 2.   
ampl _ FloatArray new: n.

"Pre-calculate amplitude values."
    i_0. [i<n] whileTrue:[
 		ampl at: i put: (  ((real at:i) squared) + ((imag at:i) squared) ).
		i_i+1.
    ].
 
"Zero out amplitudes for frequencies below 20 Hz."
    low _ ( numSamples * 20.0 / samplingRate ) asInteger.
    ( low>=0 && low<numSamples ) ifFalse:[self error: 'Assertion Failed'].
    i_low. [ i >= 0 ] whileTrue:[
        	ampl at:i put: 0.0.
		i _ i - 1.
    ].

   "In the following loop, i1 is the left side of the window and i2 is the right side."
    maxWeight _ -1.0.
    centerFreq _ -1.0.

    i2 _ windowSize. [ i2 < n ] whileTrue:[
        i1 _ i2 - windowSize.
        moment _ 0.0.
        weight _ 0.0.

        i _ i1. [ i <= i2 ] whileTrue:[
            weight _ weight + (ampl at: i).
            moment _ moment + (i * (ampl at: i)).
            i _ i + 1.
        ].

        ( weight > maxWeight ) ifTrue:[
            maxWeight _ weight.
            centerFreq _ moment / weight.
        ].
        i2 _ i2 + 1.
    ].

   ^ samplingRate * centerFreq / numSamples! !

!FrequencyFinderModule methodsFor: 'processing' stamp: 'efc 5/15/2002 03:16'!
processBuffer: buf

	| freq freqString data |
	data _ fft transformDataFrom: buf startingAt: 1.
	freq _ self fundamentalFromFFT: data.

	freqString _ (freq roundTo: 0.01) printString padded: #left to: 8 with: (Character space).
	display contents: (freqString, ' hz').! !

!FrequencyFinderModule methodsFor: 'initializing' stamp: 'efc 12/1/2001 16:27'!
initialize

super initialize.
fft setSize: 512.
self buildDisplay.! !

!FrequencyFinderModule methodsFor: 'private' stamp: 'efc 5/15/2002 02:44'!
buildDisplay
display _ StringMorph new 
	contents: '-----.-- hz';
	color: (GraphicAnalyzerMorph uiColors at: #text);
	font: ((TextConstants at: #DefaultFixedTextStyle) fontOfSize: 24).! !

!FrequencyFinderModule methodsFor: 'fft' stamp: 'efc 12/1/2001 21:20'!
setFFTSize 

super setFFTSize.! !

!FrequencyFinderModule methodsFor: 'finding frequency' stamp: 'efc 12/1/2001 15:44'!
absorbBin: newFFTBin into: peaks
	"Either extend an exiting peak record with the given fftBin, or create a new peak record. An fftBin is a pair <binIndex><binValue>. A peak record is an array of fftBins (sorted by increasing index) spanning a contiguous range of indices in the orignal FFT data."

	| binIndex peakBins |
	binIndex _ newFFTBin first.
	1 to: peaks size do: [:i |
		peakBins _ peaks at: i.
		binIndex = (peakBins first first - 1) ifTrue: [
			peaks at: i put: (Array with: newFFTBin), peakBins.
			^ peaks].
		binIndex = (peakBins last first + 1) ifTrue: [
			peaks at: i put: peakBins, (Array with: newFFTBin).
			^ peaks]].
	"newFFTBin was not contiguous with an existing peak; add it"
	^ peaks copyWith: (Array with: newFFTBin)! !

!FrequencyFinderModule methodsFor: 'finding frequency' stamp: 'efc 12/1/2001 15:45'!
centerFreqFromBins: bins
	"Compute the center frequency constituting the weighted average of the given collection of bins."

	| peakTotal weightedCenterIndex v freq |
	peakTotal _ 0.
	weightedCenterIndex _ 0.
	bins do: [:bin |
		v _ bin last.
		peakTotal _ peakTotal + v.
		weightedCenterIndex _ weightedCenterIndex + (bin first * v)].

	weightedCenterIndex _ weightedCenterIndex asFloat / peakTotal.
	weightedCenterIndex <= 1.0
		ifTrue: [freq _ 0.0]
		ifFalse: [freq _ (weightedCenterIndex - 1.0) * samplingRate asFloat / fft n].

	^ Array with: freq with: peakTotal! !

!FrequencyFinderModule methodsFor: 'finding frequency' stamp: 'efc 12/1/2001 15:45'!
energyForOvertonesOf: f in: freqs
	"Answer the total energy in the frequency records that match overtones of f."

	| e thisF thisError |
	e _ 0.
	freqs do: [:freqRec |
		thisF _ freqRec first.
		thisF >= f ifTrue: [
			thisError _ (thisF / f) fractionPart.
			(thisError < 0.06 or: [thisError > 0.94]) ifTrue: [e _ e + freqRec last]]].
	^ e
! !

!FrequencyFinderModule methodsFor: 'finding frequency' stamp: 'efc 12/1/2001 15:45'!
findFundamentalIn: freqPeaks

	| totalEnergy acceptThreshold bestEnergy possibleF e bestF |
	totalEnergy _ freqPeaks inject: 0 into: [:t :peak | t + peak last].
	acceptThreshold _ (0.9 * totalEnergy) truncated.
	bestEnergy _ 0.
	#(1 2 3) do: [:divisor |
		freqPeaks do: [:p |
			possibleF _ p first / divisor.
			e _ self energyForOvertonesOf: possibleF in: freqPeaks.
			"e > acceptThreshold" false ifTrue: [^ possibleF].  "look no further!!"
			e > bestEnergy ifTrue: [
				bestEnergy _ e.
				bestF _ possibleF]]].
	^ bestF
! !

!FrequencyFinderModule methodsFor: 'finding frequency' stamp: 'efc 12/1/2001 15:46'!
fundamentalFromFFT: fftData
	"Answer the fundamental frequency from the given FFT spectrum data."

	| maxPeak fftBins v peaks freqs f |
	maxPeak _ 0.
	fftBins _ OrderedCollection new: fftData size.
	2 to: fftData size do: [:i |
		v _ (fftData at: i) truncated.
		(v * 10) > maxPeak ifTrue: [
			v > maxPeak ifTrue: [maxPeak _ v].
			fftBins add: (Array with: i with: v)]].

	peaks _ Array new.
	fftBins do: [:b |
		(b last * 10) > maxPeak ifTrue: [
		peaks _ self absorbBin: b into: peaks]].
	peaks sort: [:e1 :e2 | e1 first first < e2 first first].

	freqs _ peaks collect: [:peak | self centerFreqFromBins: peak].
	f _ self findFundamentalIn: freqs.
	^ f
! !


!GraphicAnalyzerModule class methodsFor: 'as yet unclassified' stamp: 'efc 11/28/2001 15:43'!
getName
"What would I like to be called (like in a menu)"
^self name! !

!GraphicAnalyzerModule class methodsFor: 'as yet unclassified' stamp: 'efc 11/29/2001 14:47'!
includeInAnalyzerMenu
"Should I be instantiated from menus?"

"Its ok for this to be true here because the menu system only looks for my subclasses."
^true! !


!FFTModule class methodsFor: 'as yet unclassified' stamp: 'efc 11/28/2001 16:53'!
includeInAnalyzerMenu
^false! !


!FrequencyFinderModule class methodsFor: 'as yet unclassified' stamp: 'efc 11/29/2001 15:10'!
getName
^'Frequency Finder'! !

!FrequencyFinderModule class methodsFor: 'as yet unclassified' stamp: 'efc 11/29/2001 15:08'!
includeInAnalyzerMenu 
^true! !


!GraphicAnalyzerMorph methodsFor: 'stepping' stamp: 'efc 11/28/2001 16:15'!
step

	"update the record light and level meter"
	soundInput isRecording
		ifTrue: [statusLight color: Color yellow]
		ifFalse: [statusLight color: Color gray].

	"let the module process data"
	activeModule processBuffersFrom: soundInput! !

!GraphicAnalyzerMorph methodsFor: 'stepping' stamp: 'efc 11/25/2001 16:21'!
stepTime
^0! !

!GraphicAnalyzerMorph methodsFor: 'stepping' stamp: 'efc 11/25/2001 16:23'!
stopStepping
	"Turn off recording."

	super stopStepping.
	soundInput stopRecording.
! !

!GraphicAnalyzerMorph methodsFor: 'initialization' stamp: 'efc 12/11/2001 22:59'!
initialize
super initialize.

"sound input"
soundInput _ SoundInputStream new samplingRate: 22050.

"ui setup"
uiColors _ self class uiColors.
self layoutPolicy: TableLayout new.
self hResizing: #shrinkWrap; vResizing: #shrinkWrap.
self listDirection: #topToBottom.
self color: (uiColors at: #background).
self borderColor: (uiColors at: #border); borderWidth:3; borderRaised.
self layoutInset: 3; cellInset: 3.
self addButtonRow.

"default module"
activeModule _ SignalModule new initialize; setSamplingRate: (soundInput samplingRate).
self resetDisplay.
activeModule when: #resetDisplay send: #resetDisplay to: self.


! !

!GraphicAnalyzerMorph methodsFor: 'menu and buttons' stamp: 'efc 8/20/2002 17:16'!
setModule
	"Set the module."

	| aMenu choice modules |
	aMenu _ CustomMenu new title: 'display type (currently ', (activeModule getName), ')'.
	modules _ GraphicAnalyzerModule allSubclasses.
	modules do:[: each| 
		(each includeInAnalyzerMenu) ifTrue:[aMenu add: (each getName) action: each]
	].
	choice _ aMenu startUp.
	choice ifNil: [^ self].

	"Instantiate the new module, and listen for #resetDisplay events"
	activeModule _ choice new initialize; setSamplingRate: (soundInput samplingRate).
	self resetDisplay.
	activeModule when: #resetDisplay send: #resetDisplay to: self.

! !

!GraphicAnalyzerMorph methodsFor: 'menu and buttons' stamp: 'efc 11/29/2001 10:01'!
setSamplingRate
	"Set the sampling rate to be used for incoming sound data."

	| aMenu rate on |
	aMenu _ CustomMenu new title:
		'Sampling rate (currently ', soundInput samplingRate printString, ')'.
	#(11025 22050 44100) do:[:r | aMenu add: r printString action: r].
	rate _ aMenu startUp.
	rate ifNil: [^ self].
	on _ soundInput isRecording.
	self stop.
	soundInput samplingRate: rate.
	activeModule setSamplingRate: rate.	
	on ifTrue: [self start].
! !

!GraphicAnalyzerMorph methodsFor: 'menu and buttons' stamp: 'efc 11/28/2001 14:51'!
start
	"Start displaying sound data."

	soundInput bufferSize: (activeModule bufferSize).
	soundInput startRecording.
! !

!GraphicAnalyzerMorph methodsFor: 'menu and buttons' stamp: 'efc 11/28/2001 14:46'!
stop
	"Stop displaying sound data."

	soundInput stopRecording.
! !

!GraphicAnalyzerMorph methodsFor: 'ui building' stamp: 'efc 12/11/2001 23:01'!
addButtonRow

	| r |
	r _ Morph new layoutPolicy: TableLayout new;
		color: (Color transparent);
		listDirection: #leftToRight;
		vResizing: #spaceFill; hResizing: #spaceFill.
	r addMorphBack: (self buttonName: 'Menu' action: #invokeMenu).
	r addMorphBack: (Morph new extent: 4@1; color: Color transparent).
	r addMorphBack: (self buttonName: 'Start' action: #start).
	r addMorphBack: (Morph new extent: 4@1; color: Color transparent).
	r addMorphBack: (self buttonName: 'Stop' action: #stop).
	r addMorphBack: (Morph new extent: 12@1; color: Color transparent).
	r addMorphBack: self makeStatusLight.
	r addMorphBack: self makeLevelSlider.
	r addMorphBack: self makeLevelMeter.
	self addMorphBack: r.! !

!GraphicAnalyzerMorph methodsFor: 'ui building' stamp: 'efc 11/25/2001 16:26'!
buttonName: aString action: aSymbol

	^ SimpleButtonMorph new
		target: self;
		label: aString;
		actionSelector: aSymbol
! !

!GraphicAnalyzerMorph methodsFor: 'ui building' stamp: 'efc 5/15/2002 02:42'!
invokeMenu
	"Invoke the settings menu."

	| aMenu selector |
	aMenu _ CustomMenu new.
	aMenu addList:	#(
		('set sampling rate'		setSamplingRate)
		('set display type'		setModule)).		
	activeModule addCustomMenuItemsTo: aMenu.
	selector _ aMenu startUp.
	selector ifNil:[ ^self ].
	
	"See if the activeModule wants to implement this; otherwise, I will try it."
	(activeModule respondsTo: selector) 
		ifTrue:[	activeModule perform: selector ] 
		ifFalse:[ self perform: selector ].

! !

!GraphicAnalyzerMorph methodsFor: 'ui building' stamp: 'efc 12/11/2001 23:01'!
makeLevelMeter

	| outerBox |
	outerBox _ RectangleMorph new extent: 125@14; color: Color lightGray.
	levelMeter _ Morph new extent: 2@10; color: Color yellow.
	levelMeter position: outerBox topLeft + (2@2).
	outerBox addMorph: levelMeter.
	^ outerBox
! !

!GraphicAnalyzerMorph methodsFor: 'ui building' stamp: 'efc 12/11/2001 22:58'!
makeLevelSlider
	| levelSlider r |
	levelSlider _ SimpleSliderMorph new color: color;
				 extent: 100 @ 2;
				 target: soundInput;
				 actionSelector: #recordLevel:;
				 adjustToValue: soundInput recordLevel.
	r _ Morph new layoutPolicy: TableLayout new;
				 color: Color transparent;
				 layoutInset: 10 @ 5;
				 listDirection: #leftToRight;
				 wrapCentering: #center;
				 cellPositioning: #leftCenter;
				 hResizing: #shrinkWrap;
				 vResizing: #rigid;
				 height: 24.
	r addMorphBack: (StringMorph new contents: '0 '; color: (uiColors at: #text)).
	r addMorphBack: levelSlider.
	r addMorphBack: (StringMorph new contents: '10'; color: (uiColors at: #text)).
	^ r! !

!GraphicAnalyzerMorph methodsFor: 'ui building' stamp: 'efc 11/25/2001 16:29'!
makeStatusLight

	| s |
	statusLight _ RectangleMorph new extent: 24@19.
	statusLight color: Color gray.
	s _ StringMorph contents: 'On'.
	s position: statusLight center - (s extent // 2).
	statusLight addMorph: s.
	^ statusLight
! !

!GraphicAnalyzerMorph methodsFor: 'events' stamp: 'efc 11/29/2001 09:50'!
resetDisplay
	| on |
	on _ soundInput isRecording.
	self stop.
	display ifNotNil:[ display delete ].
	display _ activeModule getDisplayMorph.
	self addMorphBack: display.
	self extent: 10@10.
	on ifTrue: [self start].
! !


!GraphicAnalyzerMorph class methodsFor: 'ui colors' stamp: 'efc 12/1/2001 16:51'!
darkColorScheme
| d |

d_ Dictionary new.

d	at: #background put: (Color gray: 0.25);
	at: #border put: (Color lightGray);
	at: #display put: (Color red);
	at: #text put: (Color white).

^d! !

!GraphicAnalyzerMorph class methodsFor: 'ui colors' stamp: 'efc 12/1/2001 16:54'!
uiColors

^self darkColorScheme.
! !


!NoteFinderModule methodsFor: 'interface' stamp: 'efc 12/1/2001 16:10'!
addCustomMenuItemsTo: aMenu

super addCustomMenuItemsTo: aMenu.
aMenu add: 'calibrate A' action: #setCalibrationFreq.! !

!NoteFinderModule methodsFor: 'interface' stamp: 'efc 8/20/2002 17:29'!
buildDisplay
| zeroLineMorph meterArea |
display _ Morph new.
display layoutPolicy: TableLayout new;
	color: (GraphicAnalyzerMorph uiColors at: #background);
	hResizing: #shrinkWrap; vResizing: #shrinkWrap.

textDisplay_ StringMorph new 
	contents: '--- - --.- cents';
	color: (GraphicAnalyzerMorph uiColors at: #text);
	font: ((TextConstants at: #DefaultFixedTextStyle) fontOfSize: 24).

meterArea _ Morph new.
	meterArea layoutPolicy: TableLayout new;
		color: (GraphicAnalyzerMorph uiColors at: #background);
		hResizing: #shrinkWrap; vResizing: #shrinkWrap;
		listDirection: #horizontal.
	
	meter_ SimpleSliderMorph new 
		extent: (textDisplay width)@2;
		color: (GraphicAnalyzerMorph uiColors at: #background);
		minVal: -0.5;
		maxVal: 0.5.

		zeroLineMorph _ RectangleMorph new 
			color: Color red;
			borderWidth: 0;
			width: 2; 
			height: meter height;
			center: meter center.
		
		meter addMorph: zeroLineMorph.
	meterArea addMorphBack: (StringMorph new 
		contents: 'flat ';
		color: (GraphicAnalyzerMorph uiColors at: #text);
		font: ((TextConstants at: #DefaultFixedTextStyle) fontOfSize: 16) ).
	meterArea addMorphBack: meter.
	meterArea addMorphBack: (StringMorph new 
		contents: ' sharp';
		color: (GraphicAnalyzerMorph uiColors at: #text);
		font: ((TextConstants at: #DefaultFixedTextStyle) fontOfSize: 16) ).

display addMorphBack: meterArea.
display addMorphBack: textDisplay.
! !

!NoteFinderModule methodsFor: 'interface' stamp: 'efc 11/30/2001 15:26'!
setCalibrationFreq

	"Set the frequency for middle-A. Normally 440 hz."

	| aMenu freq |
	aMenu _ CustomMenu new title: 'Frequency for middle-A (currently ', calibrationFreq printString, ')'.
	((435 to: 445) collect: [:n | n]) do:[:r | aMenu add: r printString action: r].
	freq _ aMenu startUp.
	freq ifNil: [^ self].

	calibrationFreq _ freq.
! !

!NoteFinderModule methodsFor: 'processing' stamp: 'efc 8/20/2002 18:01'!
processBuffer: buf 
	| freq n noteNumber difference distance direction noteName displayString data |
	data _ fft transformDataFrom: buf startingAt: 1.
	freq _ self fundamentalFromFFT: data.
	n _ 12 * (freq / (calibrationFreq / 8 ) log: 2).
	noteNumber _ n rounded.
	difference _ noteNumber - n.
	distance _ ((difference abs * 100 roundTo: 0.1) printString)
				padded: #left
				to: 4
				with: Character space.
	difference positive
		ifTrue: [direction _ '-'].
	difference negative
		ifTrue: [direction _ '+'].
	difference isZero
		ifTrue: [direction _ ' '].
	noteName _ #('A  ' 'B-b' 'B  ' 'C  ' 'D-b' 'D  ' 'E-b' 'E  ' 'F  ' 'G-b' 'G  ' 'A-b' ) 
		atWrap: noteNumber + 1 .
	displayString _ ' ', noteName , ' ' , direction, distance , ' ', 'cents'.
	" Weight the meter so that it moves more smoothly "
	lastValue _ ( (lastValue * 8) + ((difference negated) * 2) ) / 10 asFloat.
	meter adjustToValue: lastValue.
	textDisplay contents: displayString! !

!NoteFinderModule methodsFor: 'initializing' stamp: 'efc 8/20/2002 18:02'!
initialize
	super initialize.
	"Default is A=440"
	calibrationFreq _ 440.
	lastValue _ 0.! !


!NoteFinderModule class methodsFor: 'as yet unclassified' stamp: 'efc 8/20/2002 17:48'!
getName
^'Note Finder / Tuner'! !


!SignalModule methodsFor: 'initializing' stamp: 'efc 11/29/2001 14:31'!
initialize

display _ GraphMorph new.
display extent: (400 + (2 * display borderWidth))@128.
display data: (Array new: 100 withAll: 0).
display color: (Color r: 0.8 g: 1.0 b: 1.0).! !

!SignalModule methodsFor: 'processing' stamp: 'efc 11/28/2001 16:11'!
bufferSize
^(display width - (2 * display borderWidth))! !

!SignalModule methodsFor: 'processing' stamp: 'efc 11/28/2001 09:25'!
processBuffer: buf

	display data: buf; changed! !


!SignalModule class methodsFor: 'as yet unclassified' stamp: 'efc 11/28/2001 15:54'!
getName
^'Signal'! !


!SonogramModule methodsFor: 'initializing' stamp: 'efc 11/30/2001 15:05'!
initialize

super initialize.
self buildDisplay.
! !

!SonogramModule methodsFor: 'processing' stamp: 'efc 11/29/2001 14:27'!
processBuffer: buf

	| data |
	data _ fft transformDataFrom: buf startingAt: 1.
	data _ data collect: [:v | v sqrt].  "square root compresses dynamic range"
	data /= 400.0.
	display plotColumn: (data copyFrom: 1 to: data size // 1).! !

!SonogramModule methodsFor: 'fft' stamp: 'efc 12/1/2001 21:21'!
setFFTSize 

(super setFFTSize) ifNil:[^nil].
self buildDisplay.
self trigger: #resetDisplay.
! !

!SonogramModule methodsFor: 'private' stamp: 'efc 12/18/2001 03:57'!
buildDisplay

	| h w zeros |
	h _ fft n // 2.
	h _ h min: 512 max: 64.
	w _ 400.
	display _
		Sonogram new
			extent: w@h
			minVal: 0.0
			maxVal: 1.0
			scrollDelta: w.
	zeros _ Array new: display height withAll: 0.
	display width timesRepeat: [display plotColumn: zeros].! !

!SonogramModule methodsFor: 'private' stamp: 'efc 12/1/2001 16:00'!
processBuffersFrom: aSoundInputStream

	| bufCount |
	"at small buffer sizes we have to update the sonogram in
	 batches or we may get behind; shoot for 8 updates/second"
	bufCount _ ( samplingRate / (8 * self bufferSize)) truncated max: 1.
	[bufCount > 0 and: [aSoundInputStream bufferCount > 0]] whileTrue: [
		self processBuffer: (aSoundInputStream nextBufferOrNil)].! !


!SonogramModule class methodsFor: 'as yet unclassified' stamp: 'efc 11/29/2001 14:47'!
getName
^'Sonogram'! !

!SonogramModule class methodsFor: 'as yet unclassified' stamp: 'efc 11/29/2001 11:17'!
includeInAnalyzerMenu
^true! !


!SpectrumModule methodsFor: 'initializing' stamp: 'efc 11/30/2001 15:07'!
initialize

	super initialize.
	self buildDisplay.! !

!SpectrumModule methodsFor: 'processing' stamp: 'efc 11/29/2001 11:15'!
processBuffer: buf

	| data |
	data _ fft transformDataFrom: buf startingAt: 1.
	display data: data; changed! !

!SpectrumModule methodsFor: 'fft' stamp: 'efc 12/1/2001 21:23'!
setFFTSize 

(super setFFTSize) ifNil:[^nil].
self buildDisplay.
self trigger: #resetDisplay.
! !

!SpectrumModule methodsFor: 'private' stamp: 'efc 11/30/2001 15:07'!
buildDisplay

	display _ GraphMorph new.
	display extent: ((fft n // 2) + (2 * display borderWidth))@128.
	display data: (Array new: fft n // 2 withAll: 0).
! !


!SpectrumModule class methodsFor: 'as yet unclassified' stamp: 'efc 11/29/2001 14:46'!
getName
^'Spectrum'! !

!SpectrumModule class methodsFor: 'as yet unclassified' stamp: 'efc 11/28/2001 16:36'!
includeInAnalyzerMenu
^true! !

