SystemOrganization addCategory: #MethodAnnotations!


Object subclass: #AnnotatedMethodPuller
	instanceVariableNames: 'name cachedPulledItems shouldCache '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MethodAnnotations'!

AnnotatedMethodPuller class
	instanceVariableNames: ''!


!AnnotatedMethodPuller commentStamp: 'ads 11/5/2003 17:12' prior: 0!
This is a class

 depends on Andreas Raab's MethodAnnotations and (optionally) Roel Wuyts' SystemChangeNotifications. Here's what it does:

It can go through the image and 'pull' items from any class-side method that includes an annotation saying <register: #(nameOfRegistry)>.

If SystemChangeNotifications is installed, the registry can cache the pulled items, so that they don't need to be recomputed every time you ask for them. (Right now this requires that all the annotated methods have a selector ending in 'ToRegister'; we could fix this if we had a better notification system.)

All registries are kept in a dictionary in a class variable, keyed by the name of the registry, so that people can push items into, say, the Flaps registry even if the Flaps aren't in the image.

You can turn off pulling for any particular registry, if you don't want to live with the performance hit of searching through all the classes in the system.

You can turn off caching for any particular registry, if you don't trust the notification system or don't want to live with the restriction of forcing people to name their methods 'somethingsToRegister'.
!

!AnnotatedMethodPuller methodsFor: 'as yet unclassified' stamp: 'ads 11/5/2003 12:38'!
asArray
	self needsToRecomputePulledItems ifTrue: [cachedPulledItems _ self recomputePulledItems].
	^ cachedPulledItems asArray! !

!AnnotatedMethodPuller methodsFor: 'as yet unclassified' stamp: 'ads 11/5/2003 12:20'!
canCache
	^ self systemChangeNotifier notNil! !

!AnnotatedMethodPuller methodsFor: 'as yet unclassified' stamp: 'ads 11/5/2003 12:26'!
classRemovedEvent: anEvent
	self resetCache.! !

!AnnotatedMethodPuller methodsFor: 'as yet unclassified' stamp: 'ads 11/5/2003 12:24'!
initializeNotifications
	self systemChangeNotifier ifNotNilDo: [:scn |
		scn uniqueInstance
			notify: self ofSystemChangesOfItem: #method using: #methodChangedEvent: ;
			notify: self ofSystemChangesOfItem: #class change: #Removed using: #classRemovedEvent:].! !

!AnnotatedMethodPuller methodsFor: 'as yet unclassified' stamp: 'ads 11/5/2003 17:02'!
initializeWithName: aString
	name _ aString.
	self initializeNotifications.! !

!AnnotatedMethodPuller methodsFor: 'as yet unclassified' stamp: 'ads 11/5/2003 12:25'!
methodChangedEvent: anEvent
	(anEvent item selector endsWith: 'ToRegister') ifTrue: [self resetCache].! !

!AnnotatedMethodPuller methodsFor: 'as yet unclassified' stamp: 'ads 11/5/2003 16:58'!
name
	^ name! !

!AnnotatedMethodPuller class methodsFor: 'as yet unclassified' stamp: 'ads 11/5/2003 17:02'!
named: aString
	^ self new
		initializeWithName: aString;
		yourself! !

!AnnotatedMethodPuller methodsFor: 'as yet unclassified' stamp: 'ads 11/5/2003 12:16'!
needsToRecomputePulledItems
	^ cachedPulledItems isNil or: [self shouldCache not or: [self canCache not]]! !

!AnnotatedMethodPuller methodsFor: 'as yet unclassified' stamp: 'ads 11/5/2003 12:13'!
recomputePulledItems
	| newItems |
	newItems _ OrderedCollection new.
	self systemNavigation allClassesDo: [:c |
		c selectorsWithProperty: #register do: [:s :registryNames |
			((registryNames collect: [:n | n asString]) includes: self name) ifTrue: [
				(c perform: s) ifNotNilDo: [:valuesForThisMethod | newItems addAll: valuesForThisMethod].
			].
		].
	].
	^ newItems! !

!AnnotatedMethodPuller methodsFor: 'as yet unclassified' stamp: 'ads 11/5/2003 12:26'!
resetCache
	cachedPulledItems _ nil.! !

!String methodsFor: '*methodAnnotations' stamp: 'ads 10/6/2003 16:07'!
selector
	^ self asSymbol! !

!Behavior methodsFor: '*methodAnnotations' stamp: 'ads 10/31/2003 13:04'!
selectorsWithProperty: aSymbol do: aBlock
	| metaclass |
	metaclass _ self class.
	metaclass selectorsDo: [:s |
		((metaclass compiledMethodAt: s) valueOfProperty: aSymbol) ifNotNilDo: [:p | aBlock value: s value: p].
	].
! !

!AnnotatedMethodPuller methodsFor: 'as yet unclassified' stamp: 'ads 11/5/2003 12:27'!
shouldCache
	^ shouldCache! !

!AnnotatedMethodPuller methodsFor: 'as yet unclassified' stamp: 'ads 11/5/2003 12:27'!
shouldCache: aBoolean
	shouldCache _ aBoolean.! !

!AnnotatedMethodPuller methodsFor: 'as yet unclassified' stamp: 'ads 11/5/2003 12:30'!
systemChangeNotifier
	^ self class environment at: #SystemChangeNotifier ifAbsent: [nil]! !
