'From Squeak3.7alpha of ''11 September 2003'' [latest update: #5595] on 3 December 2003 at 5:34:33 pm'!
"Change Set:		KCP-0113-t-TestForObsoleteNewAndInitializeMethods
Date:			3 Decembre 2003
Author:			Noury Bouraqadi
For Squeak:		3.7a-5595

Tests that the new-initialize pattern is used in all classes. Tests that there is no redefinition of the new method performing initialization and that there is no empty initialize method (except in ProtoObject). Obsolets tests 102-t, 103-t, 104-t, 106-t, 107-t and 112-t"!

TestCase subclass: #ObsoleteMethodTest
	instanceVariableNames: 'targetClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tests-Compatibility'!

!ObsoleteMethodTest commentStamp: 'Noury Bouraqadi 10/22/2003 09:32' prior: 0!
Abstract class.
Subclasses should implement #obsoleteMethodSelector!

ObsoleteMethodTest subclass: #EmptyInitializeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tests-Compatibility'!
ObsoleteMethodTest subclass: #NewInitializeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tests-Compatibility'!

!NewInitializeTest commentStamp: 'ar 9/16/2003 01:16' prior: 0!
NewInitializeTest is a test which is run on all classes in the system to detect obsolete or otherwise considered invalid implementations of #new. It provides a set of patterns against which all implementations of #new are run. To add a new pattern provide a method beginning with #obsolete such as #obsoleteStrangeNewTest.!


!ObsoleteMethodTest methodsFor: 'printing'!
printOn: aStream 
	aStream
		nextPutAll: self class printString;
		nextPutAll: '>>#';
		nextPutAll: testSelector.
	self targetClass ifNotNil: 
			[aStream
				nextPutAll: ' for ';
				nextPutAll: self targetClass name]! !

!ObsoleteMethodTest methodsFor: 'private'!
copySuperedMethod: aMethod 
	"Create a copy of a potentially supered method"

	| copy nLits lastLit |
	copy := aMethod copyWithTrailerBytes: #(0 0 0 0).	"regular"
	nLits := aMethod numLiterals.
	nLits > 0 
		ifTrue: 
			[lastLit := copy literalAt: nLits.
			(lastLit isVariableBinding and: [lastLit value == self class]) 
				ifTrue: [copy literalAt: nLits put: nil -> self targetClass class]].
	^copy! !

!ObsoleteMethodTest methodsFor: 'private' stamp: 'Noury Bouraqadi 9/16/2003 12:48'!
patterns
	"Answer all the methods implementing one of the obsolete #new methods"
	| list |
	list := IdentityDictionary new.
	self class selectorsAndMethodsDo:[:sel :meth|
		(sel beginsWith: 'obsolete') ifTrue:[
			list at: sel put: (self copySuperedMethod: meth)]].
	^list! !

!ObsoleteMethodTest methodsFor: 'tests'!
testObsoleteMethod
	"Test if the implementation of obsoleteMethod matches any of the patterns defined in this test"

	| aMethod |
	aMethod := self targetClass compiledMethodAt: self class obsoleteMethodSelector
				ifAbsent: [^self].	"allow the test to succeed if (for example) the method was removed"
	self patterns 
		keysAndValuesDo: [:obsoleteSelector :obsoleteMethod | self deny: aMethod = obsoleteMethod]! !

!ObsoleteMethodTest methodsFor: 'accessing' stamp: 'Noury Bouraqadi 9/16/2003 15:48'!
targetClass
	^targetClass! !

!ObsoleteMethodTest methodsFor: 'accessing' stamp: 'Noury Bouraqadi 9/16/2003 15:48'!
targetClass: aClass 
	targetClass := aClass! !


!EmptyInitializeTest methodsFor: 'patterns' stamp: 'Noury Bouraqadi 9/16/2003 13:01'!
obsoleteAnsweringSuperInitialize

	^super initialize	! !

!EmptyInitializeTest methodsFor: 'patterns' stamp: 'Noury Bouraqadi 9/16/2003 12:59'!
obsoleteEmptyInitialize
	"Should be implemented only by Object"! !

!EmptyInitializeTest methodsFor: 'patterns' stamp: 'Noury Bouraqadi 9/16/2003 13:00'!
obsoleteSuperInitialize

	super initialize	! !


!NewInitializeTest methodsFor: 'patterns' stamp: 'ar 9/16/2003 00:16'!
obsoleteSelfBasicNewInitialize
	"This pattern is obsoleted by Object class>>new"
	^self basicNew initialize! !

!NewInitializeTest methodsFor: 'patterns' stamp: 'ar 9/16/2003 00:23'!
obsoleteSuperBasicNewInitialize
	"This pattern is obsoleted by Object class>>new"
	^super basicNew initialize! !

!NewInitializeTest methodsFor: 'patterns' stamp: 'ar 9/16/2003 00:22'!
obsoleteSuperNew
	"This is pointless unless specifically documented"
	^super new! !

!NewInitializeTest methodsFor: 'patterns' stamp: 'ar 9/16/2003 00:16'!
obsoleteSuperNewInitialize
	"This pattern is obsoleted by Object class>>new"
	^super new initialize! !

!NewInitializeTest methodsFor: 'patterns' stamp: 'ar 9/16/2003 00:19'!
obsoleteSuperNewInitializeWithTemp
	"This pattern is obsoleted by Object class>>new"
	| temp |
	temp := super new.
	temp initialize.
	^temp! !

!NewInitializeTest methodsFor: 'patterns' stamp: 'ar 9/16/2003 00:20'!
obsoleteSuperNewInitializeWithTempDirectly
	"This pattern is obsoleted by Object class>>new"
	| temp |
	temp := super new initialize.
	^temp! !

!NewInitializeTest methodsFor: 'patterns' stamp: 'ar 9/16/2003 00:17'!
obsoleteSuperNewInitializeYourself
	"This pattern is obsoleted by Object class>>new"
	^(super new) initialize; yourself! !


!ObsoleteMethodTest class methodsFor: 'Instance Creation'!
selector: aSelector targetClass: aClass 
	^(self new)
		setTestSelector: aSelector;
		targetClass: aClass;
		yourself! !

!ObsoleteMethodTest class methodsFor: 'Building Suites' stamp: 'Noury Bouraqadi 12/3/2003 16:57'!
addToSuiteFromSelectors: suite
	suite addTests: self buildSuite tests! !

!ObsoleteMethodTest class methodsFor: 'Building Suites'!
buildSuite
	"Construct this test suite from all classes implementing the obsoleteMethod"
	| suite  |

	suite := self suiteClass named: self name asString.
	self isAbstract ifTrue: [^suite].
	self classesToTest do:[:aClass|
		(aClass includesSelector: self obsoleteMethodSelector) ifTrue:[
			suite addTest: 
				(self 
					selector: self testMethodSelector 
					targetClass: aClass).
		].
	].
	^suite! !

!ObsoleteMethodTest class methodsFor: 'Building Suites' stamp: 'Noury Bouraqadi 10/22/2003 09:44'!
buildSuiteFromSelectors
	^self buildSuite! !

!ObsoleteMethodTest class methodsFor: 'Building Suites' stamp: 'Noury Bouraqadi 9/16/2003 12:58'!
classesToTest
	^Smalltalk allClasses asArray sort:[:cls1 :cls2| cls1 name <= cls2 name]! !

!ObsoleteMethodTest class methodsFor: 'Building Suites' stamp: 'Noury Bouraqadi 9/16/2003 12:53'!
obsoleteMethodSelector
	self subclassResponsibility! !

!ObsoleteMethodTest class methodsFor: 'Building Suites'!
testMethodSelector
	^#testObsoleteMethod! !

!ObsoleteMethodTest class methodsFor: 'Testing'!
isAbstract
	^(self class whichClassIncludesSelector: #obsoleteMethodSelector) == ObsoleteMethodTest class! !


!EmptyInitializeTest class methodsFor: 'Building Suites'!
classesToTest
	"Its Ok if Object has a method #initialize"

	^super classesToTest copyWithout: ProtoObject! !

!EmptyInitializeTest class methodsFor: 'Building Suites' stamp: 'Noury Bouraqadi 9/16/2003 12:56'!
obsoleteMethodSelector
	^#initialize! !


!NewInitializeTest class methodsFor: 'Building Suites' stamp: 'Noury Bouraqadi 9/16/2003 15:07'!
classesToTest
	^super classesToTest collect: [:aClass| aClass class]! !

!NewInitializeTest class methodsFor: 'Building Suites' stamp: 'Noury Bouraqadi 9/16/2003 12:53'!
obsoleteMethodSelector
	^#new! !


!ObsoleteMethodTest reorganize!
('printing' printOn:)
('private' copySuperedMethod: patterns)
('tests' testObsoleteMethod)
('accessing' targetClass targetClass:)
!

