Stream subclass: #AsyncFileStream
	instanceVariableNames: 'asyncFile bufferSize buffer readIndex filePosition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AsyncFileStream'!

!AsyncFileStream methodsFor: 'private buffering' stamp: 'cds 9/11/2005 15:34'!
bufferHasData
	^buffer notNil and: [readIndex <= buffer size and: [(buffer at: readIndex) ~= 0]]! !

!AsyncFileStream methodsFor: 'private buffering' stamp: 'cds 9/12/2005 23:37'!
bufferUpTo: count
	| result |
	result := buffer copyFrom: readIndex to: count.
	readIndex := count + 1.
	^result! !

!AsyncFileStream methodsFor: 'private buffering' stamp: 'cds 9/12/2005 09:01'!
filePosition
	^filePosition ifNil: [filePosition := 0]! !

!AsyncFileStream methodsFor: 'private buffering' stamp: 'cds 9/12/2005 09:01'!
filePosition: anObject
	filePosition := anObject! !

!AsyncFileStream methodsFor: 'private buffering' stamp: 'cds 9/12/2005 09:02'!
loadBuffer
	| n tmpBuffer |
	tmpBuffer := String new: self bufferSize.
	asyncFile 
		primReadStart: asyncFile fileHandle
		fPosition: self filePosition
		count: self bufferSize.
	
	[asyncFile waitForCompletion.
	n := asyncFile 
				primReadResult: asyncFile fileHandle
				intoBuffer: tmpBuffer
				at: 1
				count: self bufferSize.
	n = -1] 
			whileTrue.
	"loop while busy in case the semaphore had excess signals"
	n = -2 ifTrue: [^self error: 'asynchronous read operation failed'].
	buffer := tmpBuffer copyFrom: 1 to: n.
	readIndex := 1.
	self filePosition: self filePosition + n! !

!AsyncFileStream methodsFor: 'private buffering' stamp: 'cds 9/11/2005 15:31'!
nextFromBuffer
	| result |
	result := buffer at: readIndex.
	readIndex := readIndex + 1.
	^result! !

!AsyncFileStream methodsFor: 'private buffering' stamp: 'cds 9/12/2005 23:33'!
restOfBufferAndReset
	| tmpBuffer |
	tmpBuffer := buffer copyFrom: readIndex to: buffer size.
	readIndex := nil.
	buffer := nil.
	^tmpBuffer! !

!AsyncFileStream methodsFor: 'private buffering' stamp: 'cds 9/12/2005 23:33'!
wholeBufferAndReset
	| tmpBuffer |
	readIndex := nil.
	tmpBuffer := buffer.
	buffer := nil.
	^tmpBuffer! !


!AsyncFileStream methodsFor: 'accessing' stamp: 'cds 9/12/2005 11:42'!
atEnd
	self bufferHasData ifFalse: [self loadBuffer].
	^buffer isNil or: [buffer isEmpty]! !

!AsyncFileStream methodsFor: 'accessing' stamp: 'cds 9/12/2005 09:00'!
bufferSize
	^bufferSize ifNil: [bufferSize := 1024]! !

!AsyncFileStream methodsFor: 'accessing' stamp: 'cds 9/12/2005 09:00'!
bufferSize: anObject
	bufferSize := anObject! !

!AsyncFileStream methodsFor: 'accessing' stamp: 'cds 9/9/2005 20:30'!
close
	asyncFile close! !

!AsyncFileStream methodsFor: 'accessing' stamp: 'cds 9/12/2005 16:45'!
name
	^asyncFile fileName! !

!AsyncFileStream methodsFor: 'accessing' stamp: 'cds 9/11/2005 15:28'!
next
	self bufferHasData ifFalse: [self loadBuffer].
	^self bufferHasData ifTrue: [self nextFromBuffer] ifFalse: [nil]! !

!AsyncFileStream methodsFor: 'accessing' stamp: 'cds 9/12/2005 23:37'!
next: anInteger 
	"Answer the next anInteger number of objects accessible by the
	receiver. "

	| ws leftToRead t |
	self bufferHasData ifFalse: [self loadBuffer].
	(readIndex = 1 and: [buffer size = anInteger]) 
		ifTrue: [^self wholeBufferAndReset].
	leftToRead := anInteger.
	ws := WriteStream on: String new.
	"whole buffers at a time"
	[leftToRead > (buffer size - readIndex) and: [self atEnd not]] 
		whileTrue: [t :=  self restOfBufferAndReset.
			leftToRead := leftToRead - t size.
			ws nextPutAll: t.
			self loadBuffer].
	self atEnd ifFalse: [ws nextPutAll: (self bufferUpTo: leftToRead)].
	^ws contents! !

!AsyncFileStream methodsFor: 'accessing' stamp: 'cds 9/12/2005 16:39'!
size
	^asyncFile fileSize! !


!AsyncFileStream methodsFor: 'initialize-release' stamp: 'cds 9/13/2005 08:18'!
openReadOnlyFileNamed: filename
	asyncFile := AsyncFile new open: filename forWrite: false.
	asyncFile ifNil: [^nil].
	^self! !


!AsyncFileStream methodsFor: 'misc' stamp: 'cds 9/12/2005 16:45'!
mimeType
		^MIMEDocument guessTypeFromName: self name.
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

AsyncFileStream class
	instanceVariableNames: ''!

!AsyncFileStream class methodsFor: 'instance creation' stamp: 'cds 9/9/2005 20:33'!
new
	^self basicNew! !

!AsyncFileStream class methodsFor: 'instance creation' stamp: 'cds 9/13/2005 08:17'!
readOnlyFileNamed: aFilename
	^self new openReadOnlyFileNamed: aFilename! !


!AsyncFileStream class methodsFor: 'testing' stamp: 'cds 9/13/2005 08:15'!
isAFileNamed: fileName
	"Answer true if a file of the given name exists."
	| f |
	f := self readOnlyFileNamed: fileName.
	f ifNil: [^ false].
	f close.
	^ true
! !


TestCase subclass: #AsyncFileStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AsyncFileStream'!

!AsyncFileStreamTest methodsFor: 'private' stamp: 'cds 9/9/2005 19:46'!
fileName
	^'testAsyncFileStream'! !

!AsyncFileStreamTest methodsFor: 'private' stamp: 'cds 9/9/2005 20:32'!
makeFile: name containing: aString
	| f |
	f := FileDirectory default forceNewFileNamed: name.
	f nextPutAll: aString.
	f close.! !


!AsyncFileStreamTest methodsFor: 'tests' stamp: 'cds 9/12/2005 09:13'!
testAtEnd
	| s |
	self makeFile: self fileName containing: 'abc'.
	s := AsyncFileStream readOnlyFileNamed: self fileName.
	self deny: s atEnd.
	self assert: s next = $a.
	self deny: s atEnd.
	self assert: s next = $b.
	self deny: s atEnd.
	self assert: s next = $c.
	self assert: s atEnd.
	s close! !

!AsyncFileStreamTest methodsFor: 'tests' stamp: 'cds 9/9/2005 20:30'!
testNext
	| s |
	self makeFile: self fileName containing: 'abc'.
	s := AsyncFileStream readOnlyFileNamed: self fileName.
	self assert: s next = $a.
	self assert: s next = $b.
	self assert: s next = $c.
	s close! !

!AsyncFileStreamTest methodsFor: 'tests' stamp: 'cds 9/12/2005 16:52'!
testNextColon
	| s |
	self makeFile: self fileName containing: 'abc'.
	s := AsyncFileStream readOnlyFileNamed: self fileName.
	self assert: (s next: 2) = 'ab'.
	s close.

	s := AsyncFileStream readOnlyFileNamed: self fileName.
	self assert: (s next: 100) = 'abc'.
	s close.
! !

!AsyncFileStreamTest methodsFor: 'tests' stamp: 'cds 9/12/2005 23:22'!
testNextColonSameSize
	| s contents |
 	contents :=  'abc
def
ghi
jklmnopq
1111111222222'.
	self makeFile: self fileName containing: contents.
	s := AsyncFileStream readOnlyFileNamed: self fileName.
	s bufferSize: 5.
	self assert: (s next: 5) = 'abc
d'.
	self assert: (s next: 5) = 'ef
gh'.
	s close.
! !

!AsyncFileStreamTest methodsFor: 'tests' stamp: 'cds 9/12/2005 17:02'!
testNextColonSmallBuffer
	| s contents |
 	contents :=  'abc
def
ghi
jklmnopq
1111111222222'.
	self makeFile: self fileName containing: contents.
	s := AsyncFileStream readOnlyFileNamed: self fileName.
	s bufferSize: 5.
	self assert: (s next: 4) = 'abc
'.
	self assert: (s next: 6) = 'def
gh'.
	s close.
! !

!AsyncFileStreamTest methodsFor: 'tests' stamp: 'cds 9/12/2005 09:03'!
testReadSmallBuffer
	| s contents actual |
	contents :=  'abc
def
ghi
jklmnopq
1111111222222'.
	self makeFile: self fileName containing: contents.
	s := AsyncFileStream readOnlyFileNamed: self fileName.
	s bufferSize: 5.
	actual := ReadStream on: contents.
	[actual atEnd] whileFalse: [self assert: s next = actual next].
	s close! !

!AsyncFileStreamTest methodsFor: 'tests' stamp: 'cds 9/12/2005 16:17'!
testSize
	| s |
	self makeFile: self fileName containing: 'abc'.
	s := AsyncFileStream readOnlyFileNamed: self fileName.
	self assert: s size = 3.
	s close! !



ComancheModule subclass: #AsyncModFile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AsyncFileStream'!

!AsyncModFile methodsFor: 'validation' stamp: 'cds 9/12/2005 12:14'!
validateForStack: anArray problems: problems

	super validateForStack: anArray problems: problems.
	anArray 
		detect: [ :ea | ea isDoc ] 
		ifNone: [problems add: 'File module requires a Doc module']! !


!AsyncModFile methodsFor: 'processing' stamp: 'cds 9/13/2005 08:31'!
processHttp
	| fullFilePath method fs |
	method := ModCore method.
	(#(#GET #POST ) includes: method)
		ifFalse: [^ false].
	fullFilePath := ModDoc fullFilePath.
	(fs := AsyncFileStream readOnlyFileNamed: fullFilePath)
		ifNil: [^ false].
	self processSubModules
		ifTrue: [fs close.
			^ true].
	HttpResponse
		current: (HttpResponse fromStream: fs).
	^ true! !
